#ifndef FS_H
#define FS_H

#include <string>
#include <vector>

// So as not to interfere with normal p4 exit codes (which, as far as I can tell are undocumented),
// we start our error codes at 1000. Success is still signaled by FS_OK (=0)
enum EResult {
	FS_BUSY				= -1,
	FS_OK				= 0,
	FS_HELP,
	FS_ABORTED			= 1000,
	FS_ERROR_ARGUMENT,
	FS_ERROR_NO_SYNC,
	FS_ERROR_MEMORY,
	FS_ERROR_INIT,
	FS_ERROR_SYNC,
	FS_ERROR_IO,
	FS_ERROR_P4,
	FS_ERROR_RESYNC
};

const char* cFileType[];

typedef void		(*SyncStatCallbackFunc) (int inFileCount, long long inSyncSize);
typedef void		(*FileStatCallbackFunc) (const char* inDepotFile, const char* inClientFile, const char* inAction, int inRev, long long inFileSize);
typedef void		(*BufferCallbackFunc) (long long inWriteSize);
typedef void		(*ClobberCallbackFunc) (const std::string& inClobberFile);

EResult				Sync(int argc, char **argv, const char* inToolName, SyncStatCallbackFunc inSyncStatCallback = NULL, FileStatCallbackFunc inFileStatCallback = NULL, BufferCallbackFunc inReadCallback = NULL, BufferCallbackFunc inWriteCallback = NULL, ClobberCallbackFunc inClobberCallback = NULL);
void				AbortSync();
void				Finalize();

					// Helper function to force-sync writeable files that couldn't be clobbered. Can only be called after Sync() has been called first
EResult				ForceSync(const std::vector<std::string>& inFiles, const char* inToolName, SyncStatCallbackFunc inSyncStatCallback = NULL, FileStatCallbackFunc inFileStatCallback = NULL, BufferCallbackFunc inReadCallback = NULL, BufferCallbackFunc inWriteCallback = NULL, ClobberCallbackFunc inClobberCallback = NULL);

int					GetFileSpecCount();
const char*			GetErrorMessage();
const char*			GetExtendedErrorMessage();
bool				DoScriptedOutput();
int					GetNewlinesDecoded();

void				Print(const char* inFormat, ...);
void				Log(const char* inFormat, ...);

#endif