#ifndef FS_FILESYS_H
#define FS_FILESYS_H

#include "zlib.h"

class FileWriter;

class FsFileSys : public FileSys
{
public:
					FsFileSys(FileSysType inType, long long inFileSize);
	virtual			~FsFileSys();

	virtual void    Open(FileOpenMode inMode, Error* ioError);
	virtual void    Write(const char* inBuf, int inLen, Error* ioError);
	virtual int     Read(char* outBuf, int inLen, Error* ioError);
	virtual void    Close(Error* ioError);
	virtual int     Stat();
	virtual int     StatModTime();
	virtual void    Truncate(Error* ioError);
	virtual void    Unlink(Error* ioError = 0);
	virtual void    Rename(FileSys* inTarget, Error* ioError);
	virtual void    Chmod(FilePerm inPerms, Error* ioError);
	virtual void	ChmodTime(Error* ioError);
	
	static int		GetNewlinesDecoded()						{ return mNewlinesDecoded; }
	
private:
	int				DecodeText(const char* inReadBuffer, int& ioReadLength, unsigned char* inWriteBuffer, int inWriteLength, Error* ioError);
	int				DecodeGzip(const char* inReadBuffer, int& ioReadLength, unsigned char* inWriteBuffer, int inWriteLength, Error* ioError);
	int				DecodeUtf16(const char* inReadBuffer, int& ioReadLength, unsigned char* inWriteBuffer, int inWriteLength, Error* ioError);
	int				DecodeRaw(const char* inReadBuffer, int& ioReadLength, unsigned char* inWriteBuffer, int inWriteLength, Error* ioError);

	long long		mFileSize;
	FileWriter*		mFileWriter;
	z_stream		mZStream;
	int				mZStreamState;
	bool			mDecodeFailed;
	bool			mBomWritten;
	int				mLineCount;
	int				mUtf8ByteCount;
	int				mCurCodePoint;
	static int		mNewlinesDecoded;
};

#endif